# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 03.11.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 10, task 6.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation

import os

#import numpy as np
import cv2
import matplotlib.pyplot as plt
from skimage import morphology as morph # pylint: disable=import-error
#import seaborn as sns # pylint: disable=import-error,unused-import

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def main():
  """main"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 10')
  print('Task 6')
  print('-'*80)

  fig_num = 0
  image_dir = '../../images'
  write_dir = 'results/images'

  page_image = cv2.imread(os.path.join(image_dir, 'morf_test.png'), cv2.IMREAD_GRAYSCALE)
  fig_num = plot_image(page_image, fig_num,
                       write_file=os.path.join(write_dir, 't06_original.png'))

  # Try with binary thresholding, does not work well.
  _, page_thresh = cv2.threshold(page_image, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)
  fig_num = plot_image(page_thresh, fig_num,
                       write_file=os.path.join(write_dir, 't06_thresh.png'))

  # Extract background and subtract it
  selem = morph.disk(5)
  background = morph.closing(page_image, selem)
  fig_num = plot_image(background, fig_num,
                       write_file=os.path.join(write_dir, 't06_bg.png'))

  background_removed = background - page_image
  fig_num = plot_image(background_removed, fig_num,
                       write_file=os.path.join(write_dir, 't06_bg_removed.png'))

  # Which is the same (per def) as bottom hat (black top hat)
  bhat = morph.black_tophat(page_image, selem)
  fig_num = plot_image(bhat, fig_num,
                       write_file=os.path.join(write_dir, 't06_bhat.png'))

  # Threshold the result
  _, bhat_thresh = cv2.threshold(bhat, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)
  fig_num = plot_image(bhat_thresh, fig_num,
                       write_file=os.path.join(write_dir, 't06_bhat_thresh.png'))

  # and remove small elements ("pixel dust")
  small_removed = morph.remove_small_objects(bhat_thresh.astype(bool),
                                             min_size=5,
                                             connectivity=8)
  fig_num = plot_image(small_removed, fig_num,
                       write_file=os.path.join(write_dir, 't06_small_removed.png'))

  # Finally, invert the colors
  fig_num = plot_image(1 - small_removed, fig_num,
                       write_file=os.path.join(write_dir, 't06_final.png'))





  plt.show()


if __name__ == "__main__":
  main()
